'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var PropTypes = _interopDefault(require('prop-types'));
var Measure = _interopDefault(require('react-measure'));
var reactSpring = require('react-spring');

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var Grid =
/*#__PURE__*/
function (_React$Component) {
  _inheritsLoose(Grid, _React$Component);

  function Grid() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _React$Component.call.apply(_React$Component, [this].concat(args)) || this;
    _this.state = {
      width: 0,
      height: 0,
      active: undefined,
      lastActive: undefined
    };

    _this.preventScroll = function (e) {
      if (!_this.props.lockScroll) {
        _this.state.active && _this.onClick(undefined);
        _this.clicked = false;
      }
    };

    _this.onClick = function (key) {
      return _this.setState(function (state) {
        return {
          lastActive: state.active,
          active: state.active === key ? undefined : key
        };
      }, function () {
        return _this.clicked = true;
      });
    };

    _this.resizeOuter = function (props) {
      return _this.setState({
        widthOuter: props.client.width,
        heightOuter: props.client.height
      });
    };

    _this.resizeInner = function (props) {
      return _this.setState({
        width: props.client.width,
        height: props.client.height
      });
    };

    _this.update = function (_ref) {
      var key = _ref.key,
          x = _ref.x,
          y = _ref.y,
          width = _ref.width,
          height = _ref.height;
      var active = _this.state.active === key;
      return {
        opacity: _this.state.active && !active ? 0 : 1,
        x: active ? _this.outerRef.scrollLeft : x,
        y: active ? _this.outerRef.scrollTop : y,
        width: active ? _this.state.width : width,
        height: active ? _this.state.heightOuter : height
      };
    };

    return _this;
  }

  var _proto = Grid.prototype;

  _proto.componentDidUpdate = function componentDidUpdate() {
    this.clicked = false;
  };

  _proto.render = function render() {
    var _this2 = this;

    var _this$props = this.props,
        children = _this$props.children,
        columns = _this$props.columns,
        margin = _this$props.margin,
        occupySpace = _this$props.occupySpace,
        impl = _this$props.impl,
        config = _this$props.config,
        data = _this$props.data,
        keys = _this$props.keys,
        heights = _this$props.heights,
        inactiveDelay = _this$props.inactiveDelay,
        lockScroll = _this$props.lockScroll,
        rest = _objectWithoutPropertiesLoose(_this$props, ["children", "columns", "margin", "occupySpace", "impl", "config", "data", "keys", "heights", "inactiveDelay", "lockScroll"]);

    var _this$state = this.state,
        lastActive = _this$state.lastActive,
        active = _this$state.active;
    var column = 0;
    var columnHeights = new Array(columns).fill(0);
    var displayData = data.map(function (child, i) {
      var index = occupySpace ? columnHeights.indexOf(Math.min.apply(Math, columnHeights)) : column++ % columns;
      var width = _this2.state.width / columns - margin / (1 - 1 / (columns + 1));
      var left = width * index;
      var offset = (index + 1) * margin;
      var top = columnHeights[index] + margin;
      var height = typeof heights === 'function' ? heights(child) : heights;
      columnHeights[index] += height + margin;
      return {
        x: margin ? left + offset : left,
        y: top,
        width: width,
        height: height,
        key: keys(child),
        object: child
      };
    });
    return React.createElement(Measure, {
      client: true,
      innerRef: function innerRef(r) {
        return _this2.outerRef = r;
      },
      onResize: this.resizeOuter
    }, function (_ref2) {
      var measureRef = _ref2.measureRef;
      return React.createElement("div", _extends({
        ref: measureRef,
        style: _extends({
          position: 'relative',
          width: '100%',
          height: '100%'
        }, _this2.props.style, {
          overflow: lockScroll ? active ? 'hidden' : 'auto' : 'auto'
        })
      }, rest, {
        onScroll: _this2.preventScroll,
        onWheel: _this2.preventScroll,
        onTouchMove: _this2.preventScroll
      }), React.createElement(Measure, {
        client: true,
        innerRef: function innerRef(r) {
          return _this2.innerRef = r;
        },
        onResize: _this2.resizeInner
      }, function (_ref3) {
        var measureRef = _ref3.measureRef;
        return React.createElement("div", {
          ref: measureRef,
          style: {
            position: 'relative',
            width: '100%',
            overflow: 'hidden',
            height: Math.max.apply(Math, columnHeights) + margin
          }
        }, React.createElement(reactSpring.Transition, {
          native: true,
          delay: _this2.clicked && !active ? inactiveDelay : 0,
          items: displayData,
          keys: function keys(d) {
            return d.key;
          },
          from: {
            opacity: 0
          },
          leave: {
            opacity: 0
          },
          enter: _this2.update,
          update: _this2.update,
          impl: impl,
          config: config
        }, displayData.map(function (child, i) {
          return function (_ref4) {
            var opacity = _ref4.opacity,
                x = _ref4.x,
                y = _ref4.y,
                width = _ref4.width,
                height = _ref4.height;
            var key = child.key;
            return React.createElement(reactSpring.animated.div, {
              style: {
                opacity: opacity,
                width: width,
                height: height,
                position: 'absolute',
                willChange: 'transform, width, height, opacity',
                zIndex: lastActive === key || active === key ? 1000 : i,
                transform: reactSpring.interpolate([x, y], function (x, y) {
                  return "translate3d(" + x + "px," + y + "px, 0)";
                })
              },
              children: children(child.object, active === key, function () {
                return _this2.onClick(key);
              })
            });
          };
        })));
      }));
    });
  };

  return Grid;
}(React.Component);

Grid.defaultProps = {
  occupySpace: true,
  columns: 3,
  margin: 0
};
process.env.NODE_ENV !== "production" ? Grid.propTypes = {
  occupySpace: PropTypes.bool,
  columns: PropTypes.number,
  margin: PropTypes.number,
  data: PropTypes.array,
  keys: PropTypes.func,
  heights: PropTypes.func
} : void 0;

module.exports = Grid;
